import { useState, useEffect, useRef, useCallback } from 'react';
import { fieldSubscriptionItems, formSubscriptionItems, createForm } from 'final-form';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var all = fieldSubscriptionItems.reduce(function (result, key) {
  result[key] = true;
  return result;
}, {});

var subscriptionToInputs = function subscriptionToInputs(subscription) {
  return fieldSubscriptionItems.map(function (key) {
    return Boolean(subscription[key]);
  });
};

var eventValue = function eventValue(event) {
  if (!event.target) {
    return event;
  } else if (['checkbox', 'radio'].includes(event.target.type)) {
    return event.target.checked;
  }

  return event.target.value;
};

var useField = function useField(name, form, subscription) {
  if (subscription === void 0) {
    subscription = all;
  }

  var autoFocus = useRef(false);

  var _useState = useState({}),
      state = _useState[0],
      setState = _useState[1];

  useEffect(function () {
    return form.registerField(name, function (newState) {
      if (autoFocus.current) {
        autoFocus.current = false;
        setTimeout(function () {
          return newState.focus();
        });
      }

      setState(newState);
    }, subscription);
  }, [name].concat(subscriptionToInputs(subscription)));

  var _ref = state || {},
      blur = _ref.blur,
      change = _ref.change,
      focus = _ref.focus,
      value = _ref.value,
      meta = _objectWithoutPropertiesLoose(_ref, ["blur", "change", "focus", "value"]);

  delete meta.name; // it's in input

  return {
    input: {
      name: name,
      value: value || '',
      onBlur: function onBlur() {
        return state.blur();
      },
      onChange: function onChange(event) {
        return state.change(eventValue(event));
      },
      onFocus: function onFocus() {
        if (state.focus) {
          state.focus();
        } else {
          autoFocus.current = true;
        }
      }
    },
    meta: meta
  };
};

var all$1 = formSubscriptionItems.reduce(function (result, key) {
  result[key] = true;
  return result;
}, {});
/**
 * Converts { active: true, data: false, ... } to `[true, false, false, ...]`.
 */

var subscriptionToInputs$1 = function subscriptionToInputs(subscription) {
  return formSubscriptionItems.map(function (key) {
    return Boolean(subscription[key]);
  });
};

var useFormState = function useFormState(form, subscription) {
  if (subscription === void 0) {
    subscription = all$1;
  }

  var _useState = useState(function () {
    return form.getState();
  }),
      state = _useState[0],
      setState = _useState[1];

  useEffect(function () {
    return form.subscribe(setState, subscription);
  }, [form].concat(subscriptionToInputs$1(subscription)));
  return state;
};

var useForm = function useForm(_ref) {
  var subscription = _ref.subscription,
      config = _objectWithoutPropertiesLoose(_ref, ["subscription"]);

  var form = useRef(); // https://reactjs.org/docs/hooks-faq.html#how-to-create-expensive-objects-lazily

  var getForm = function getForm() {
    if (!form.current) {
      form.current = createForm(config);
    }

    return form.current;
  };

  var state = useFormState(getForm(), subscription);
  var handleSubmit = useCallback(function (event) {
    if (event) {
      if (typeof event.preventDefault === 'function') {
        event.preventDefault();
      }

      if (typeof event.stopPropagation === 'function') {
        event.stopPropagation();
      }
    }

    return getForm().submit();
  }, []);
  return _extends({}, state, {
    form: getForm(),
    handleSubmit: handleSubmit
  });
};

export { useField, useForm, useFormState };
